import { Component, OnInit } from "@angular/core";
import { FormArray, FormBuilder, FormControl, FormGroup, Validators } from "@angular/forms";
import { TranslateService } from "@ngx-translate/core";
import { catchError } from "rxjs/operators";
import { SettingsDTO } from "src/app/models/config/settings.dto";
import { ErrorDTO } from "src/app/models/error/error-dto";
import { ServiceErrorCode } from "src/app/models/error/service-error-code.enum";
import { GuiService } from "src/app/services/gui/gui.service";
import { SettingsService } from "src/app/services/settings/settings.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { ToastService } from "src/app/services/toast/toast.service";
import { forkJoin, throwError } from "rxjs";
import { TechnicianConfig } from "../../../../../models/config/technician-config.model";
import { TechnicianConfigQuery } from "../../../../../models/graphql/query.model";
import { Queries } from "../../../../../models/graphql/queries.model";
import { DataService } from "../../../../../services/data/data.service";
import { TechnicianConfigMutationResult } from "../../../../../models/graphql/result.model";
import { Mutations } from "../../../../../models/graphql/mutations.model";
import { AuthService } from "../../../../../services/auth/auth.service";
import { Table } from "primeng/table";

type PingUrl = {
  id: number;
  value: string;
};

type TechnicianId = {
  id: number;
  value: string;
};

@Component({
  selector: "app-settings",
  templateUrl: "./settings.component.html",
  styleUrls: ["./settings.component.scss"]
})
export class SettingsComponent implements OnInit {

  testMailForm!: FormGroup;
  technicianConfigForm: FormGroup = this.formBuilder.group({
    helpCommandReferenceUrl: [null, [Validators.required]],
    waitForRestart: [null, [Validators.required]],
    waitForShutdown: [null, [Validators.required]],
    cacheMaxMessages: [null, [Validators.required]]
  });
  pingUrls: PingUrl[] = [];
  clonedPingUrls: { [s: number]: PingUrl } = {};
  technicianIds: TechnicianId[] = [];
  clonedTechnicianIds: { [s: number]: TechnicianId } = {};
  data: SettingsDTO = {
    webVersion: "",
    apiVersion: "",
    configPath: "",
    webBaseURL: "",
    apiBaseURL: "",

    tokenExpireTime: 0,
    refreshTokenExpireTime: 0,

    mailUser: "",
    mailPort: 0,
    mailHost: "",
    mailTransceiver: "",
    mailTransceiverAddress: ""
  };

  config: TechnicianConfig = {
    helpCommandReferenceUrl: "",
    waitForRestart: 0,
    waitForShutdown: 0,
    cacheMaxMessages: 0,
    pingURLs: [],
    technicianIds: []
  };

  constructor(
    private dataService: DataService,
    private settingsService: SettingsService,
    private spinnerService: SpinnerService,
    private guiService: GuiService,
    private formBuilder: FormBuilder,
    private toastService: ToastService,
    private translate: TranslateService,
    private authService: AuthService,
    private spinner: SpinnerService
  ) {
  }

  ngOnInit(): void {
    this.spinnerService.showSpinner();
    this.initForms();

    forkJoin([
      this.guiService.getSettings().pipe(catchError(error => {
        this.spinnerService.hideSpinner();
        return throwError(() => error);
      })),
      this.dataService.query<TechnicianConfigQuery>(Queries.technicianConfigQuery)
    ]).subscribe(data => {
      this.data = data[0];
      this.data.webVersion = this.settingsService.getWebVersion()?.getVersionString() ?? "0.0.0";
      this.data.apiBaseURL = this.settingsService.getApiURL();
      if (!this.data.apiBaseURL.endsWith("/")) {
        this.data.apiBaseURL += "/";
      }

      this.config = data[1].technicianConfig;
      this.initForms();
      this.spinnerService.hideSpinner();
    });
  }

  initForms(): void {
    this.testMailForm = this.formBuilder.group({
      mail: [null, [Validators.required, Validators.email]]
    });
    this.technicianConfigForm = this.formBuilder.group({
      helpCommandReferenceUrl: [this.config.helpCommandReferenceUrl, [Validators.required]],
      waitForRestart: [this.config.waitForRestart, [Validators.required]],
      waitForShutdown: [this.config.waitForShutdown, [Validators.required]],
      cacheMaxMessages: [this.config.cacheMaxMessages, [Validators.required]]
    });
    let id = 0;
    for (const url of this.config.pingURLs ?? []) {
      this.pingUrls.push({ id: id, value: url });
      id++;
    }

    id = 0;
    for (const technicianId of this.config.technicianIds ?? []) {
      this.technicianIds.push({ id: id, value: technicianId });
      id++;
    }
  }

  testMail(): void {
    this.spinnerService.showSpinner();

    const mail = this.testMailForm.value.mail;
    if (!mail) {
      this.spinnerService.hideSpinner();
      return;
    }

    this.guiService.sendTestMail(mail)
      .pipe(catchError(error => {
        let header = this.translate.instant("admin.settings.message.error");
        let message = this.translate.instant("admin.settings.message.could_not_send_mail");

        if (error.error !== null) {
          const err: ErrorDTO = error.error;

          if (err.errorCode === ServiceErrorCode.ConnectionFailed) {
            header = this.translate.instant("admin.settings.message.connection_failed");
            message = this.translate.instant("admin.settings.message.connection_to_mail_failed");
            error.error = null;
          }

          if (err.errorCode === ServiceErrorCode.InvalidUser) {
            header = this.translate.instant("admin.settings.message.connection_failed");
            message = this.translate.instant("admin.settings.message.mail_login_failed");
            error.error = null;
          }

          if (err.errorCode === ServiceErrorCode.MailError) {
            header = this.translate.instant("admin.settings.message.send_failed");
            message = this.translate.instant("admin.settings.message.test_mail_not_send");
            error.error = null;
          }
        }

        this.spinnerService.hideSpinner();
        this.toastService.error(header, message);
        return throwError(() => error);
      }))
      .subscribe(res => {
        this.spinnerService.hideSpinner();
        this.toastService.success(this.translate.instant("admin.settings.message.success"), this.translate.instant("admin.settings.message.send_mail"));
        this.testMailForm.reset();
      });
  }

  saveTechnicianConfig() {
    this.spinner.showSpinner();
    this.dataService.mutation<TechnicianConfigMutationResult>(Mutations.updateTechnicianConfig, {
        helpCommandReferenceUrl: this.config.helpCommandReferenceUrl,
        waitForRestart: this.config.waitForRestart,
        waitForShutdown: this.config.waitForShutdown,
        cacheMaxMessages: this.config.cacheMaxMessages,
        pingURLs: this.pingUrls.filter(value => value.value).map(value => {
          return value.value;
        }),
        technicianIds: this.technicianIds.filter(value => value.value).map(value => {
          return value.value;
        })
      }
    ).pipe(catchError(err => {
      this.spinner.hideSpinner();
      this.toastService.error(this.translate.instant("admin.settings.message.technician_config_create_failed"), this.translate.instant("admin.settings.message.technician_config_create_failed_d"));
      return throwError(err);
    })).subscribe(result => {
      this.spinner.hideSpinner();
      this.toastService.success(this.translate.instant("admin.settings.message.technician_config_create"), this.translate.instant("admin.settings.message.technician_config_create_d"));
    });
  }

  pingUrlsAddNew(table: Table) {
    const id = Math.max.apply(Math, this.pingUrls.map(url => {
      return url.id ?? 0;
    })) + 1;
    const newItem = { id: id, value: "" };
    this.pingUrls.push(newItem);

    table.initRowEdit(newItem);
    const index = this.pingUrls.findIndex(l => l.id == newItem.id);
    this.pingUrlsEditInit(newItem, index);
  }

  pingUrlsEditInit(url: PingUrl, index: number) {
    this.clonedPingUrls[index] = { ...url };
  }

  pingUrlsDelete(index: number) {
    this.pingUrls.splice(index, 1);
  }

  pingUrlsEditSave(url: PingUrl, index: number) {
    if (!url.value || this.pingUrls[index] == this.clonedPingUrls[index]) {
      return;
    }

    delete this.clonedPingUrls[index];
  }

  pingUrlsEditCancel(url: PingUrl, index: number) {
    this.pingUrls[index] = this.clonedPingUrls[index];
    delete this.clonedPingUrls[index];
  }


  technicianIdsAddNew(table: Table) {
    const id = Math.max.apply(Math, this.technicianIds.map(url => {
      return url.id ?? 0;
    })) + 1;
    const newItem = { id: id, value: "" };
    this.technicianIds.push(newItem);

    table.initRowEdit(newItem);
    const index = this.technicianIds.findIndex(l => l.id == newItem.id);
    this.technicianIdsEditInit(newItem, index);
  }

  technicianIdsEditInit(url: PingUrl, index: number) {
    this.clonedTechnicianIds[index] = { ...url };
  }

  technicianIdsDelete(index: number) {
    this.technicianIds.splice(index, 1);
  }

  technicianIdsEditSave(url: PingUrl, index: number) {
    if (!url.value || this.technicianIds[index] == this.clonedTechnicianIds[index]) {
      return;
    }

    delete this.clonedTechnicianIds[index];
  }

  technicianIdsEditCancel(url: PingUrl, index: number) {
    this.technicianIds[index] = this.clonedTechnicianIds[index];
    delete this.clonedTechnicianIds[index];
  }

}
